from getpass import getpass
import os, shelve

db = None
db_path = '/eduroam/'

def openDB(args):
    global db
    global db_path

    db = shelve.open(os.path.join(db_path, 'config'), writeback=True)
    if len(db.keys()) == 0 and (args.update or args.test):
        raise EnvironmentError("Cannot update the configuration because the configuration was never initialized. "
                               "Please rerun eduroam-config without --update.")

    if  len(db.keys()) == 0: os.system('sudo chmod 777 ' + os.path.join(db_path, 'config.db'))

    db['wifi_id'] = db['wifi_id'] if 'wifi_id' in db and args.wifi_id is None else args.wifi_id
    db['email_list'] = db['email_list'] if 'email_list' in db and not args.email_list else []
    db['priority'] = db['priority'] if 'priority' in db and args.priority is None else args.priority if args.priority is not None else 0
    db['delay_time'] = db['delay_time'] if 'delay_time' in db and args.delay_time is None else args.delay_time if args.delay_time is not None else 30

    db.close()
    db = shelve.open(os.path.join(db_path, 'config'), writeback=True)


def closeDB():
    global db
    db.close()


def saveDB():
    global db
    global db_path
    db.close()
    db = shelve.open(os.path.join(db_path, 'config'), writeback=True)


def getConfig(key):
    global db
    return db[str(key)]


def setConfig(key, value):
    global db
    db[str(key)] = value


def getID():
    id = input('Eduroam ID [' + str(getConfig('wifi_id')) + ']: ')
    while id == '' and getConfig('wifi_id') is None:
        print('Eduroam ID cannot be blank. Try your school email.')
        id = input('Eduroam ID [' + str(getConfig('wifi_id')) + ']: ')

    return id


def getPW():
    pw1 = ''
    pw2 = ''
    while pw1 != pw2 or pw1 == '' and pw2 == '':
        pw1 = getpass('Eduroam Password: ')
        pw2 = getpass('Confirm Eduroam Password: ')
        if pw1 != pw2: print('Passwords do not match. Please try again.')
        elif pw1 == '' or pw2 == '': print('Password cannot be blank. Please try again.')

    return pw1


def getPriority():
    invalid = True
    while invalid:
        try:
            priority = input('Eduroam Connection Priority [' + str(getConfig('priority')) + ']: ')
            if priority == '':
                invalid = False
            else:
                priority = int(priority)
                if priority < 0: raise ValueError('Value cannot be negative.')
                invalid = False
        except Exception:
            print('The priority level was invalid. Please enter a positive integer.')

    return priority


def getEmailList():
    ans = ' '
    while ans != 'y' and ans != '':
        ans = ' '
        emails = input('List of Emails to Be Notified (separated by commas) [' + str(getConfig('email_list')) + ']: ')
        while emails == '' and getConfig('email_list') is None:
            print('List of Emails cannot be blank. You need to be notified via email if your Pi\'s IP changes.')
            emails = input('List of Emails to Be Notified (separated by commas) [' + str(getConfig('email_list')) + ']: ')
        if emails == '': break

        try:
            emails = (''.join(emails.split(' '))).split(',')
            print('Your email list is the following:\n', emails)
            while ans != '' and ans != 'y' and ans != 'n':
                ans = input('Are you satisfied with this list? (y)/n: ')
        except Exception:
            print('There is an error with your email list input. Please try again... "email1@mail.com,email2@mail.com')

    return emails


def getTimeDelay():
    invalid = True
    while invalid:
        try:
            time_delay = input('Notification Delay in Seconds [' + str(getConfig('delay_time')) + ']: ')
            if time_delay == '':
                invalid = False
            else:
                time_delay = int(time_delay)
                if time_delay < 0: raise ValueError('Value cannot be negative.')
                invalid = False
        except Exception:
            print('The delay time was invalid. Please enter a positive integer.')

    return time_delay


def getWifiCredentials(args):
    id = getID()
    pw = getPW()
    priority = getPriority()

    if id != '': setConfig('wifi_id', id)
    if priority != '': setConfig('priority', priority)
    args.wifi_pw = pw
    print()

    return args


def configNotificationService(update=False):
    emails = getEmailList() if not update else getConfig('email_list')
    time_delay = getTimeDelay() if not update else getConfig('delay_time')

    if emails != "": setConfig('email_list', emails)
    if time_delay != "": setConfig('delay_time', time_delay)


# Print iterations progress
def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = printEnd)
    # Print New Line on Complete
    if iteration == total:
        print()



from eduroam_config import wpa_supplicant_config
from eduroam_config.utils import openDB, closeDB, getConfig, getEmailList, getPW, getWifiCredentials, setConfig
from eduroam_config.eduroam_service import createService, testService
import argparse, os, time

wpa_path = '/etc/wpa_supplicant/'


def main():
    global wpa_path

    parser = argparse.ArgumentParser(description='A command line tool to configure your raspberry pi to connect to an '
                                                 'eduroam enterprise network and notify the end-user of ip changes for '
                                                 'ssh connections.\nCopy and Paste the following to be walked-through '
                                                 'the configuration process\n\n    sudo eduroam-config')
    parser.add_argument('-dt', '--delay-time', type=int, action='store',
                        help='Time delay in seconds between network connection and sending the ip-change email. It is '
                             'recommended to have at least a 30 second delay.')
    parser.add_argument('-id', '--wifi-id', type=str, action='store',
                        help='The identification (email) assocaited with your eduroam wifi credentials.')
    parser.add_argument('-pw', '--wifi-pw', action='store_true', default=False,
                        help='No argument flag. This will prompt you to reset your password safely. --update is '
                             'implicitly called with this flag.')
    parser.add_argument('-e', '--email-list', action='store_true', default=False,
                        help='No argument flag. This will prompt you to create your email list.')
    parser.add_argument('-p', '--priority', type=int, action='store',
                        help='Sets the priority of eduroam connection. Higher integers correlate to higher '
                             'priority. See here for more details. '
                             'https://www.raspberrypi.org/forums/viewtopic.php?t=42378')
    parser.add_argument('-u', '--update', action='store_true', default=False,
                        help='No argument flag. Mutes the configuration prompts and simply applies the changes specified in the call '
                             'arguments.')
    parser.add_argument('-t', '--test', action='store_true', default=False, help='Only executes a notification test for the '
                        'eduroam.service.')

    args = parser.parse_args()

    if args.wifi_pw:
        args.update = True

    if args.delay_time is not None and args.delay_time < 0:
        raise ValueError('Delay time must be a positive number')

    if args.priority is not None and args.priority < 0:
        raise ValueError('Priority must be a positive number')

    print('Setting environment...')

    if not os.path.exists('/eduroam/'):
        print('Creating directory /eduroam/...', end='\r')
        os.mkdir('/eduroam/')
        print('Creating directory /eduroam/... Done.')

    print('Opening config database...', end='\r')
    openDB(args)
    print('Opening config database... Done.\n')

    if args.email_list:
        setConfig('email_list', getEmailList())

    if args.test:
        testService()
    elif args.update:
        if args.wifi_id is not None or args.wifi_pw or args.priority is not None:
            args.wifi_pw = getPW()
            wpa_supplicant_config.configWPA(getConfig('wifi_id'), args.wifi_pw, getConfig('priority'), wpa_path)
        if args.delay_time is not None or args.email_list:
            createService(update=True)
        print('Changes to you eduroam configuration were successful. Thanks!  =(^_^)=')
    else:
        print('Beginning configuration process...')
        print('Please enter the following:')
        args = getWifiCredentials(args)
        wpa_supplicant_config.configWPA(getConfig('wifi_id'), args.wifi_pw, getConfig('priority'), wpa_path)
        time.sleep(1)
        createService(update=False)

    closeDB()


if __name__ == '__main__':
    main()



import os, time


def configWPA(id, pw, priority, path):
    wpa_struct = list()
    temp_start_index = None
    start_index = None
    end_index = None
    print('Reconfiguring wpa_suplicant.conf', end='\r')
    time.sleep(0.7)
    print('Reconfiguring wpa_suplicant.conf.', end='\r')
    time.sleep(0.7)
    print('Reconfiguring wpa_suplicant.conf..', end='\r')
    time.sleep(0.7)
    print('Reconfiguring wpa_suplicant.conf...', end='\r')
    time.sleep(0.7)
    with open(os.path.join(path, 'wpa_supplicant.conf')) as f:
        line = f.readline()
        while line:
            wpa_struct.append(line)
            if '{' in line:
                temp_start_index = len(wpa_struct) - 1

            if 'eduroam' in line:
                start_index = temp_start_index

            if start_index is not None and '}' in line:
                end_index = len(wpa_struct)

            line = f.readline()

        wpa_struct_begin = wpa_struct[:start_index]
        wpa_struct_end = wpa_struct[end_index:]

    eduroam_wpa_struct = ['network={\n',
                          '  ssid="eduroam"\n',
                          '  key_mgmt=WPA-EAP\n',
                          '  pairwise=CCMP\n',
                          '  eap=PEAP\n',
                          '  identity="' + str(id) + '"\n',
                          '  password="' + str(pw) + '"\n',
                          '  ca_cert="/etc/ssl/certs/AddTrust_External_Root.pem"\n',
                          '  phase2="auth=MSCHAPV2"\n',
                          '  priority=' + str(priority) + '\n',
                          '}\n']

    eduroam_wpa_struct_print = ['network={\n',
                          '  ssid="eduroam"\n',
                          '  key_mgmt=WPA-EAP\n',
                          '  pairwise=CCMP\n',
                          '  eap=PEAP\n',
                          '  identity="' + str(id) + '"\n',
                          '  password="****"\n',
                          '  ca_cert="/etc/ssl/certs/AddTrust_External_Root.pem"\n',
                          '  phase2="auth=MSCHAPV2"\n',
                          '  priority=' + str(priority) + '\n',
                          '}\n']

    if start_index is None:
        wpa_struct_print = wpa_struct + ['\n'] + eduroam_wpa_struct_print
        wpa_struct += ['\n'] + eduroam_wpa_struct
    else:
        wpa_struct = wpa_struct_begin + eduroam_wpa_struct + wpa_struct_end
        wpa_struct_print = wpa_struct_begin + eduroam_wpa_struct_print + wpa_struct_end

    with open(os.path.join(path, 'wpa_supplicant.conf'), 'w') as f:
        f.writelines(wpa_struct)
        print('\nReconfigured wpa_supplicant.conf:\n')
        print('\n' + '=' * 60)
        print(' ' * 20 + 'wpa_supplicant.conf' + ' ' * 25)
        print('-' * 60)
        print(''.join(wpa_struct_print))
        print('=' * 60 + '\n')



import os, subprocess, time
import eduroam_config.utils as module
from eduroam_config.utils import saveDB, configNotificationService, getConfig, printProgressBar


def testService():
    delay_time = getConfig('delay_time')
    input('\nNotification service is configured and enabled. We are going to test the service. If you have good internet'
          ' connection, you should expect to receive your test email in roughly ' + str(delay_time + 10) +
          ' seconds with your current connection IP.\n\nPress enter when ready to start test...')

    ans = ' '
    again = ' '
    while ans != '' and ans != 'y' and again != 'n':
        ans = ' '
        again = ' '
        print('Running test...')
        subprocess.call(['sudo', '-S', 'systemctl', 'start', 'eduroam.service'])
        time.sleep(1)
        print('Gathering eduroam service test data...')
        os.system('sudo -S systemctl status eduroam.service')
        t = 0
        while t <= delay_time + 10:
            time.sleep(0.01)
            printProgressBar(t, delay_time + 10, 'Time Before Notification:', str(round(t, 2)) + ' seconds', 0, 25)
            t = round(t + 0.01, 2)
        print()

        while ans != '' and ans != 'y' and ans != 'n':
            ans = input('Did you receive an email? (y)/n: ')

        while ans != 'y' and again != '' and again != 'y' and again != 'n':
            print('Gathering service status data...')
            time.sleep(1)
            os.system('sudo -S systemctl status eduroam.service')
            time.sleep(1)
            again = input('Do you want to try again? (y)/n: ')

    if ans == 'y' or ans == '':
        print('Congratulations! If you enter valid credentials, you successfully configured your Raspberry Pi to '
              'connect to eduroam and notify you! Thanks!  =(^_^)=')
    elif ans == 'n':
        print('Fear not! Sometimes running this command from your bin fails to test successfully.\nCopy and Paste '
              'this into your terminal to test the service again.\n\n    sudo eduroam-config --test\n\nIf it is '
              'still not working, Copy and Paste this into your terminal and wait patiently for ' + str(delay_time + 10) +
              ' seconds.\n\n    sudo systemctl start eduroam.service\n')

def constructServiceTemplate():
    bin = os.path.join('/'.join(os.path.abspath(module.__file__).split('/')[:-1]), 'bin')

    with open('/eduroam/eduroam.service.template', 'w') as f:
        temp = '[Unit]\n'\
                'Description=eduroam-config is dependent on this service\n'\
                'Wants=network-online.target\n'\
                'After=network-online.target\n'\
                '\n'\
                '[Service]\n'\
                'Type=simple\n'\
                'User=' + os.getenv('USER') + '\n'\
                'WorkingDirectory=' + bin + '\n'\
                'ExecStart=' + os.path.join(bin, 'notify') + '\n'\
                '\n'\
                '[Install]\n'\
                'WantedBy=multi-user.target'
        f.write(temp)


def createService(update=False):
    print('Creating notification service template...', end='\r')
    constructServiceTemplate()
    print('Creating notification service template... Done.')

    if os.path.exists('/etc/systemd/system/eduroam.service'):
        print('Previous service configuration found. Deleting...')
        os.system('sudo -S systemctl stop eduroam.service')
        os.system('sudo -S systemctl disable eduroam.service')
        time.sleep(1)
        os.system('sudo rm -r /etc/systemd/system/eduroam.*;')
        print('Service removed.')

    print('Creating new notification service eduroam.service:')
    print('\n' + '=' * 60)
    print(' ' * 20 + 'eduroam.service' + ' ' * 25)
    print('-' * 60)
    os.system('sudo -S SYSTEMD_EDITOR=tee systemctl edit --force --system eduroam.service < /eduroam/eduroam.service.template')
    time.sleep(1)
    print('\n' + '-' * 60)
    print()

    print('Enabling notification service:')
    time.sleep(1)  
    os.system('sudo -S systemctl enable eduroam.service')
    time.sleep(1)

    print('Reloading daemon...', end='\r')
    os.system('sudo -S systemctl daemon-reload')
    time.sleep(0.5)
    print('Reloading daemon... Done.')

    print('eduroam.service status:')
    os.system('sudo -S systemctl status eduroam.service')    
    print('\nPlease enter the following:')
    configNotificationService(update)
    saveDB()
    if not update: testService()






import os, setuptools


def copy_dir():
    dir_path = '/Users/benjamindaghir1/Documents/eduroam-pi/dist/mail'
    base_dir = os.path.join('', dir_path)
    for (dirpath, dirnames, files) in os.walk(base_dir):
        for f in files:
            yield os.path.join(dirpath.split('/', 1)[1], f)


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="eduroam-pi",
    version="0.5.0",
    author="Benjamin Ryan Daghir",
    author_email="daghirb@gmail.com",
    description="Networking package to connect to enterprise eduroam and obtain"
                + "IP for ssh connection.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
    entry_points = {
        'console_scripts': ['eduroam-config=eduroam_config.entry_point:main'],
    }
)
